/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.productionunit;

import java.awt.Color;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Action;
import nl.cloudfarming.client.farm.model.CropField;
import nl.cloudfarming.client.geoviewer.Layer;
import nl.cloudfarming.client.geoviewer.Palette;
import nl.cloudfarming.client.geoviewer.SingleObjectLayer;
import nl.cloudfarming.client.util.actions.OpenAction;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.util.HelpCtx;
import org.openide.util.Lookup;
import org.openide.util.Utilities;
import org.openide.util.datatransfer.ExTransferable;
import org.openide.util.datatransfer.PasteType;
import org.openide.util.lookup.Lookups;

/**
 * Node to display a {@link CropField}
 *
 * @author johan
 * @author Wytse Visser
 */
public final class CropFieldNode extends AbstractNode implements Serializable {

    private static final String CROP_FIELD_ACTION_PATH = "Actions/Cropfield";
    private static final Logger LOGGER = Logger.getLogger(CropFieldNode.class.getName());
    private final CropField cropField;

    public CropFieldNode(CropField cropField) {
        super(Children.LEAF, createLookup(cropField));
        assert cropField != null : "Should be a CropField in the lookup of a CropFieldNode";
        this.cropField = cropField;

        initNameAndIcon();
        initPropertyChangeListeners();
    }

    private static Lookup createLookup(CropField cropField) {
        return Lookups.fixed(cropField, createLayer(cropField), new CropFieldSplitHandler(cropField));
    }

    private static Layer createLayer(CropField cropField) {
        return new SingleObjectLayer(cropField.getName(), cropField, null, createPalette(cropField));
    }

    private void initNameAndIcon() {
        setName(cropField.getName());
        setDisplayName(getName());
        setIconBaseWithExtension("nl/cloudfarming/client/productionunit/draw_island16.png");
    }

    private void initPropertyChangeListeners() {
        cropField.addPropertyChangeListener(CropField.PROP_COLOR, new PaletteColorUpdater());
        cropField.addPropertyChangeListener(CropField.PROP_NAME, new NameUpdater());
    }

    public static Palette createPalette(CropField cropField) {
        Palette palette = new Palette(cropField.getColor());
        return palette;
    }

    @Override
    public PasteType getDropType(Transferable t, int action, int index) {
        return null;
    }

    @Override
    public HelpCtx getHelpCtx() {
        return new HelpCtx("nl.cloudfarming.client.cropfield.type.about");
    }

    @Override
    public Action[] getActions(boolean context) {
        List<Action> actions = new ArrayList<>();
        actions.add(new OpenAction(CropField.MIMETYPE, this));
        //add all actions registered in the layer file
        for (Action action : Utilities.actionsForPath(CROP_FIELD_ACTION_PATH)) {
            actions.add(action);
        }
        return actions.toArray(new Action[actions.size()]);
    }

    @Override
    public boolean canRename() {
        return true;
    }

    @Override
    public void setName(String s) {
        super.setName(s);
        cropField.setName(s);
    }

    @Override
    public Transferable clipboardCopy() throws IOException {
//        return super.clipboardCopy();
        //calling super.clipboardCopy prevents this method
        //from working correctly! Instead call clipboardCut
        //even when just copying this node.
        Transferable transferable = super.clipboardCopy();
        ExTransferable exTrans = ExTransferable.create(transferable);
        exTrans.put(new ExTransferable.Single((CropField.DATA_FLAVOR)) {
            @Override
            protected CropField getData() throws IOException, UnsupportedFlavorException {
                return cropField;
            }
        });
        return exTrans;
    }

    @Override
    public Transferable clipboardCut() throws IOException {
        return clipboardCopy();
    }

    private class NameUpdater implements PropertyChangeListener {

        @Override
        public void propertyChange(PropertyChangeEvent evt) {
            if (evt.getNewValue() != null && evt.getPropertyName().equals(CropField.PROP_NAME)) {
                setName((String) evt.getNewValue());
                setDisplayName(getName());
            }
        }
    }

    private class PaletteColorUpdater implements PropertyChangeListener {

        private Palette palette;

        public PaletteColorUpdater() {
            palette = getLookup().lookup(Layer.class).getPalette();
        }

        @Override
        public void propertyChange(PropertyChangeEvent evt) {
            if (evt.getNewValue() instanceof Color) {
                palette.setBaseColor(cropField.getColor());
            }
        }
    }
}
