/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.productionunit;

import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.crop.model.*;
import nl.cloudfarming.client.util.swing.FormRow;
import org.openide.util.NbBundle;

/**
 * Visual panel for crop instances. Contains a drop down for crop variety and
 * classification
 *
 * @author Wytse Visser
 */
@NbBundle.Messages({
    "CropInstanceSelectorVisual crop label=Crop:",
    "CropInstanceSelectorVisual variety label=Variety:",
    "CropInstanceSelectorVisual classification label=Classification:"})
public class CropInstanceSelectorVisual extends Container {

    private JComboBox<Crop> cropSelector;
    private DefaultComboBoxModel<Crop> cropModel;
    private JComboBox<Variety> varietySelector;
    private DefaultComboBoxModel<Variety> varietyModel;
    private Set<CropDefinition> cropDefinitions;
    private JComboBox<Classification> classificationSelector;
    private DefaultComboBoxModel<Classification> classificationModel;
    public static final String PROP_SELECTION_CROP = "cropSelection";
    public static final String PROP_SELECTION_VARIETY = "varietySelection";
    public static final String PROP_SELECTION_CLASSIFICATION = "classificationSelection";

    public CropInstanceSelectorVisual(Set<CropDefinition> cropDefinitions) {
        this.cropDefinitions = cropDefinitions;
        initComponents();
    }

    /**
     * Inits the components sets the cropSelector to default selected index {@link CropInstanceSelectorVisual#DEFAULT_SELECTED_CROP}
     */
    private void initComponents() {
        this.setLayout(new MigLayout("wrap 1, insets 0 0 0 0"));
        initCropSelector();
        initVarietySelector();
        initClassificationSelector();
    }

    private void initCropSelector() {
        cropModel = new DefaultComboBoxModel<>(getPossibleCrops());
        cropSelector = new JComboBox<>(cropModel);
        cropSelector.setActionCommand(PROP_SELECTION_CROP);
        add(new FormRow(Bundle.CropInstanceSelectorVisual_crop_label(), Bundle.CropInstanceSelectorVisual_crop_label(), cropSelector));
    }

    private void initVarietySelector() {
        varietyModel = new DefaultComboBoxModel<>(getPossibleVarieties());
        varietySelector = new JComboBox<>(varietyModel);
        varietySelector.setActionCommand(PROP_SELECTION_VARIETY);
        add(new FormRow(Bundle.CropInstanceSelectorVisual_variety_label(), Bundle.CropInstanceSelectorVisual_variety_label(), varietySelector));

        cropSelector.addActionListener(new VarietySelectorUpdater());
    }

    private void initClassificationSelector() {
        classificationModel = new DefaultComboBoxModel<>(getPossibleClassifications());
        classificationSelector = new JComboBox<>(classificationModel);
        classificationSelector.setActionCommand(PROP_SELECTION_CLASSIFICATION);
        add(new FormRow(Bundle.CropInstanceSelectorVisual_classification_label(), Bundle.CropInstanceSelectorVisual_classification_label(), classificationSelector));

        cropSelector.addActionListener(new ClassificationSelectorUpdater());
    }

    private Crop[] getPossibleCrops() {
        List<Crop> crops = new ArrayList<>();
        for (CropDefinition cropDefinition : cropDefinitions) {
            crops.add(cropDefinition.getCrop());
        }
        return crops.toArray(new Crop[crops.size()]);
    }

    Variety[] getPossibleVarieties() {
        return getSelectedCropDefinition() != null ? getSelectedCropDefinition().getVarieties().toArray(new Variety[0]) : new Variety[0];
    }

    /**
     * Checks if the selected crop is not null if so will get all the
     * Classifications for this crop and return them
     *
     * @return
     */
    Classification[] getPossibleClassifications() {
        return getSelectedCropDefinition() != null ? getSelectedCropDefinition().getClassifications().toArray(new Classification[0]) : new Classification[0];
    }

    /**
     * <p>Get the selected crop definition</p> <p>Compares the selected crop
     * definitions crop against the available crop definitions crops. returns
     * the cropDefinition which has same crop</p>
     *
     * @return
     */
    CropDefinition getSelectedCropDefinition() {
        Crop crop = getSelectedCrop();
        for (CropDefinition cropDefinition : cropDefinitions) {
            if (cropDefinition.getCrop().equals(crop)) {
                return cropDefinition;
            }
        }
        return null;
    }

    Crop getSelectedCrop() {
        return cropModel.getElementAt(cropSelector.getSelectedIndex());
    }

    /**
     * <p>Set the selected {@link CropInstance} on this panel</p> Will update
     * crop drop down with {@link CropInstance#getCrop() }</br> Will update
     * variety drop down with {@link CropInstance#getVariety()}</br> Will update
     * Classification drop down with {@link CropInstance#getClassification()}
     *
     * @param instance
     */
    public void setSelected(CropInstance instance) {
        cropSelector.setSelectedItem(instance.getCrop());
        varietySelector.setSelectedItem(instance.getVariety());
        classificationSelector.setSelectedItem(instance.getClassification());
    }

    Variety getSelectedVariety() { // TODO: Privatize
        return varietyModel.getElementAt(varietySelector.getSelectedIndex());
    }

    Classification getSelectedClassification() {
        return classificationModel.getElementAt(classificationSelector.getSelectedIndex());
    }

    /**
     * <p>Add action listeners to all the drop down boxes</p> Possible action
     * commands are</br>
     * {@link CropInstanceSelectorVisual#PROP_SELECTION_CROP} for changes on the
     * selected crop</br>
     * {@link CropInstanceSelectorVisual#PROP_SELECTION_VARIETY} for changes on
     * the selected variety</br>
     * {@link CropInstanceSelectorVisual#PROP_SELECTION_CLASSIFICATION} for
     * changes on the selected Classification
     *
     * @param listener
     */
    public void addActionListener(ActionListener listener) {
        cropSelector.addActionListener(listener);
        varietySelector.addActionListener(listener);
        classificationSelector.addActionListener(listener);
    }

    /**
     * Get the selected {@link CropInstance}
     *
     * @return
     */
    public CropInstance getSelectedInstance() {
        return new CropInstance(getSelectedCrop(), getSelectedVariety(), getSelectedClassification());
    }

    /**
     * Updates the variety model
     */
    private class VarietySelectorUpdater implements ActionListener {

        @Override
        public synchronized void actionPerformed(ActionEvent e) {
            varietyModel.removeAllElements();
            for (Variety variety : getPossibleVarieties()) {
                varietyModel.addElement(variety);
            }
        }
    }

    private class ClassificationSelectorUpdater implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            classificationModel.removeAllElements();
            for (Classification classification : getPossibleClassifications()) {
                classificationModel.addElement(classification);
            }
        }
    }
}
