/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.productionunit;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JPanel;
import javax.swing.JTextField;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.crop.model.*;
import nl.cloudfarming.client.farm.model.CropProductionUnit;
import nl.cloudfarming.client.util.swing.FormRow;
import nl.cloudfarming.client.util.swing.beanbinding.NonEmptyStringValidator;
import org.jdesktop.beansbinding.*;
import org.jdesktop.swingx.JXDatePicker;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;

/**
 * Default edit panel for {@link CropProductionUnit}
 *
 * @author Merijn Zengers
 */
@NbBundle.Messages({
    "CropProductionUnitEditPanel name label text=Name:",
    "CropProductionUnitEditPanel description label text=Description:",
    "CropProductionUnitEditPanel validity label text=Validity:",
    "CropProductionUnitEditPanal planned area label=Planned area in HA:"})
public class CropProductionUnitDetailsPanel extends JPanel {

    private BindingGroup cropProductionUnitBindingGroup = new BindingGroup();
    private ELProperty formPropName = ELProperty.create("${text_ON_ACTION_OR_FOCUS_LOST}");
    private BeanProperty fieldPropName = BeanProperty.create(CropProductionUnit.PROP_NAME);
    private BeanProperty formPropDescription = BeanProperty.create("text");
    private BeanProperty fieldPropDescription = BeanProperty.create(CropProductionUnit.PROP_DESCRIPTION);
    private BeanProperty formPropValidityStart = BeanProperty.create("date");
    private BeanProperty fieldPropStartDate = BeanProperty.create(CropProductionUnit.PROP_START_DATE);
    private BeanProperty formPropValidityEnd = BeanProperty.create("date");
    private BeanProperty fieldPropEndDate = BeanProperty.create(CropProductionUnit.PROP_END_DATE);
    private BeanProperty fieldPropPlannedArea = BeanProperty.create(CropProductionUnit.PROP_PLANNED_AREA_IN_HECTARES);
    private BeanProperty formPropPlannedArea = BeanProperty.create("text");
    private JTextField nameField;
    private JTextField descriptionField;
    private JXDatePicker startingDateField;
    private JXDatePicker endDateField;
    private JTextField plannedAreaField;
    private FormRow nameRow;
    private FormRow descriptionRow;
    private FormRow validityRow;
    private FormRow plannedAreaRow;
    private final CropProductionUnit cropProductionUnit;
    protected CropInstanceSelectorVisual cropSelector;
    private static final String NAME_LABEL = Bundle.CropProductionUnitEditPanel_name_label_text();
    private static final String DESCRIPTION_LABEL = Bundle.CropProductionUnitEditPanel_description_label_text();
    private static final String VALIDITY_LABEL = Bundle.CropProductionUnitEditPanel_validity_label_text();
    private static final String PLANNED_AREA_LABEL = Bundle.CropProductionUnitEditPanal_planned_area_label(); 

    /**
     * Create the panel with a {@link CropProductionUnit} all the input values
     * will be stored in the passed CropPoductionUnit
     *
     * @param cropProductionUnit
     */
    public CropProductionUnitDetailsPanel(CropProductionUnit cropProductionUnit) {
        this.cropProductionUnit = cropProductionUnit;
        initComponents();
        bind();
    }

    private void initComponents() {
        setLayout(new MigLayout("wrap"));
        nameField = new JTextField();
        nameRow = new FormRow(true, NAME_LABEL, NAME_LABEL, nameField);
        add(nameRow);
        descriptionField = new JTextField();
        descriptionRow = new FormRow(false, DESCRIPTION_LABEL, DESCRIPTION_LABEL, descriptionField);
        add(descriptionRow);
        startingDateField = new JXDatePicker();
        startingDateField.getEditor().setEditable(false);
        endDateField = new JXDatePicker();
        endDateField.getEditor().setEditable(false);
        validityRow = new FormRow(true, VALIDITY_LABEL, VALIDITY_LABEL, startingDateField, endDateField);
        add(validityRow);
        CropDefinitionManager cropDefinitionManager = Lookup.getDefault().lookup(CropDefinitionManager.class);
        cropSelector = new CropInstanceSelectorVisual(cropDefinitionManager.findAll());
        //Check if the cropInstance is set
        //If so set the selected crop
        //If not add the first selection as selected to CropProductionUnit
        if (cropProductionUnit.getCropInstance() != null) {
            cropSelector.setSelected(cropProductionUnit.getCropInstance());
        } else {
            cropProductionUnit.setCropInstance(cropSelector.getSelectedInstance());
        }
        cropSelector.addActionListener(new CropInstanceUpdater());
        add(cropSelector);
        plannedAreaField = new JTextField();
        plannedAreaRow = new FormRow(false, PLANNED_AREA_LABEL, PLANNED_AREA_LABEL, plannedAreaField);
        add(plannedAreaRow);
    }

    /**
     * bind form fields to CropProductionUnit properties
     */
    private void bind() {
        AutoBinding nameBinding = Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ_WRITE, cropProductionUnit, fieldPropName, nameField, formPropName);
        nameBinding.setValidator(new NonEmptyStringValidator());
        cropProductionUnitBindingGroup.addBinding(nameBinding);
        cropProductionUnitBindingGroup.addBinding(Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ_WRITE, cropProductionUnit, fieldPropDescription, descriptionField, formPropDescription));
        cropProductionUnitBindingGroup.addBinding(Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ_WRITE, cropProductionUnit, fieldPropStartDate, startingDateField, formPropValidityStart));
        cropProductionUnitBindingGroup.addBinding(Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ_WRITE, cropProductionUnit, fieldPropEndDate, endDateField, formPropValidityEnd));
        cropProductionUnitBindingGroup.addBinding(Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ_WRITE, cropProductionUnit, fieldPropPlannedArea, plannedAreaField, formPropPlannedArea));
        cropProductionUnitBindingGroup.bind();
    }

    /**
     * Updates the {@link CropProductionUnit} when a change is made in the
     * {@link CropInstanceSelectorVisual}
     */
    private class CropInstanceUpdater implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent ae) {
            cropProductionUnit.setCropInstance(cropSelector.getSelectedInstance());
        }
    }

    public CropInstanceSelectorVisual getCropSelector() {
        return cropSelector;
    }

    public JTextField getDescriptionField() {
        return descriptionField;
    }

    public JXDatePicker getEndDateField() {
        return endDateField;
    }

    public JTextField getNameField() {
        return nameField;
    }

    public JXDatePicker getStartingDateField() {
        return startingDateField;
    }

    public FormRow getDescriptionRow() {
        return descriptionRow;
    }

    /**
     * Get the name row
     * @return 
     */
    public FormRow getNameRow() {
        return nameRow;
    }

    /**
     * Get the validity row
     * @return 
     */
    public FormRow getValidityRow() {
        return validityRow;
    }
    
    /**
     * Get the planned area in HA row
     * @return 
     */
    public FormRow getPlannedAreaRow(){
        return plannedAreaRow;
    }
    
    /**
     * Get the text field containing the value of the planned area in HA 
     * @return 
     */
    public JTextField getPlannedAreaField(){
        return plannedAreaField;
    }

    /**
     * Get the {@link CropProductionUnit} this panel modifies
     *
     * @return
     */
    public CropProductionUnit getCropProductionUnit() {
        return cropProductionUnit;
    }
}
