/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.productionunit;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.farm.model.CropField;
import nl.cloudfarming.client.farm.model.CropProductionUnit;
import nl.cloudfarming.client.field.model.Field;
import nl.cloudfarming.client.model.EnterpriseIdProvider;
import org.openide.util.datatransfer.PasteType;

/**
 * <p>Class which can supply the {@link PasteType} for a {@link DataFlavor}</p>
 * <p>Currently supports {@link Field#DATA_FLAVOR} and {@link CropField#DATA_FLAVOR}</p>
 *
 * @author Merijn Zengers <Merijn.Zengers@gmail.nl>
 */
public class CropProductionUnitPasteType {

    protected static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.productionunit");

    /**
     * Get the {@link PasteType} for a {@link Transferable}
     *
     * @param transferable currently supported are {@link Field#DATA_FLAVOR} and {@link CropField#DATA_FLAVOR}
     * @param cropProductionUnit The crop production unit to perform actions on
     * based on the flavor
     * @return appropriate pasteType or null if none is found
     */
    public static PasteType getPasteType(final Transferable transferable, final CropProductionUnit cropProductionUnit) {
        if (transferable.isDataFlavorSupported(Field.DATA_FLAVOR)) {

            return new FieldPasteType(transferable, cropProductionUnit);
        } else if (transferable.isDataFlavorSupported(CropField.DATA_FLAVOR)) {

            try {
                final CropField cropField = (CropField) transferable.getTransferData(CropField.DATA_FLAVOR);

                // do not drop on current parent:
                if (cropField.getParentURI().equals(cropProductionUnit.getURI())) {
                    return null;
                }

                // do not move to other farm:
                if (!EnterpriseIdProvider.Util.isSameFarm(cropField, cropProductionUnit)) {
                    return null;
                }

                return new CropFieldPasteType(cropField, cropProductionUnit);

            } catch (UnsupportedFlavorException ex) {
                LOGGER.log(Level.WARNING, "Need a cropfield to move, {0} does not contain a cropfield.", transferable.toString());
            } catch (IOException ex) {
                LOGGER.log(Level.SEVERE, "IO exception while dropping {0} {1}", new Object[]{transferable.toString(), ex});
            }
        }


        return null;
    }

    /**
     * Paste type to add {@link Field} to {@link CropProductionUnit}
     */
    static class FieldPasteType extends PasteType {

        private CropProductionUnit cropProductionUnit;
        private Transferable transferable;

        public FieldPasteType(Transferable transferable, CropProductionUnit cropProductionUnit) {
            this.cropProductionUnit = cropProductionUnit;
            this.transferable = transferable;
        }

        @Override
        public Transferable paste() throws IOException {
            try {
                Field field = (Field) transferable.getTransferData(Field.DATA_FLAVOR);
                cropProductionUnit.addCropFieldFromField(field);
            } catch (UnsupportedFlavorException ex) {
                LOGGER.log(Level.WARNING, "Need a field to convert to cropField, {0} does not contain a field.", transferable.toString());
            }
            return null;
        }
    }

    static class CropFieldPasteType extends PasteType {

        private CropField cropField;
        private CropProductionUnit cropProductionUnit;
        
        public CropFieldPasteType(CropField cropField, CropProductionUnit cropProductionUnit){
            this.cropField = cropField;
            this.cropProductionUnit = cropProductionUnit;
        }
        
        // TODO: do not allow moving cropfields depending on its state (tasks already executed)
        @Override
        public Transferable paste() throws IOException {
            cropField.getParent().removeCropField(cropField);
            cropProductionUnit.addCropField(cropField);
            return null;
        }
    }
}
